#	Programmer:	Daniel Pozmanter
#	E-mail:		drpython@bluebottle.com
#	Note:		You must reply to the verification e-mail to get through.
#
#	Copyright 2003-2005 Daniel Pozmanter
#
#	Distributed under the terms of the GPL (GNU Public License)
#
#	DrPython is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	
#Open Imported Module Dialog

import os, os.path, sys, keyword
import wx

#*******************************************************************************************************
#Utility Functions for parsing import strings,
#and getting the path for each module.

def GetModulePath(filepath, selmodule, platformiswin):
	moduletext = selmodule
	#Special Cases:
	if selmodule == 'os.path':
		if platformiswin:
			selmodule = 'ntpath'
		elif os.name == 'mac':
			selmoudle = 'macpath'
		else:
			selmodule = 'posixpath'
			
	selectedmodule = '/' + selmodule.replace('.', '/') + '.py'
	#Handle cases like 'import wx' == import 'wx.__init__'
	selectedinitmodule = '/' + selmodule.replace('.', '/') + '/__init__.py'
	
	if os.path.exists(filepath + selectedmodule):
		return moduletext, (filepath + selectedmodule)
	if os.path.exists(filepath + selectedinitmodule):
		return moduletext, (filepath + selectedinitmodule)
	
	#Search for the file:
	for somepath in sys.path:				
		modulefile = somepath.replace('\\', '/') + selectedmodule
		initfile = somepath.replace('\\', '/') + selectedinitmodule
		if os.path.exists(modulefile):
			return moduletext, modulefile
		elif os.path.exists(initfile):
			return moduletext, initfile
						
	return moduletext, None

def ParseImportStatement(matches):
	targetarray = map(lambda x: x.strip().split(), matches)
	results = []
	for item in targetarray:							
		x = 0
		isfrom = (item[0] == 'from')
		l = len(item)
		while x < l:
			if item[x].find(',') > -1:
				y = item.pop(x)
				ya = y.split(',')
				ly = len(ya)
				counter = 0
				while counter < ly:
					if len(ya[counter]) < 1:
						ya.pop(counter)
						ly -= 1
					else:
						counter += 1
				item.extend(ya)
				l = l + len(ya) - 1
			elif item[x] == 'as':
				item.pop(x)
				try:
					item.pop(x)
					l -= 2
				except:
					l -= 1
			elif item[x] == '*':
				item.pop(x)
				l -= 1
			elif item[x] in keyword.kwlist:
				item.pop(x)
				l -= 1
			else:
				if (x > 0) and isfrom:
					a = item.pop(x)
					item.insert(x, item[0] + '.' + a)
				x += 1
		results.append(item)
	return results


#*******************************************************************************************************


class drOpenImportedModuleDialog(wx.Dialog):
	def __init__(self, parent, modulelist, point=(0, 0), size=(250, 300)):
		wx.Dialog.__init__(self, parent, -1, "Open Imported Module", point, size, wx.DEFAULT_DIALOG_STYLE | wx.MAXIMIZE_BOX | wx.THICK_FRAME | wx.RESIZE_BORDER)
		
		self.parent = parent
			
		self.ID_MODULES = 101
		self.ID_TXT_MODULE = 102
		
		self.ID_OK = 111
		self.ID_CANCEL = 112
		
		#/Constants
		
		#Components:

		self.txtModule = wx.TextCtrl(self, self.ID_TXT_MODULE, '', (0, 0), (250, -1))

		self.modules = modulelist

		self.listModules = wx.ListBox(self, self.ID_MODULES, (0, 0), (250, 250), modulelist)
	
		self.btnOk = wx.Button(self, self.ID_OK, "  &Ok  ")
		
		self.btnOk.SetDefault()
		self.txtModule.SetFocus()
								
		self.btnCancel = wx.Button(self, self.ID_CANCEL, "  &Cancel  ")
		
		#/Components
		
		#Sizer:
						
		self.theSizer = wx.BoxSizer(wx.VERTICAL)
	
		self.textSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.textSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.textSizer.Add(self.txtModule, 1, wx.EXPAND)
		self.textSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)
		
		self.listSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.listSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.listSizer.Add(self.listModules, 1, wx.EXPAND)
		self.listSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)
		
		self.commandSizer = wx.BoxSizer(wx.HORIZONTAL)
		
		self.commandSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.commandSizer.Add(self.btnCancel, 0, wx.SHAPED | wx.ALIGN_LEFT)
		self.commandSizer.Add(wx.StaticText(self, -1, '  '), 1, wx.EXPAND)
		self.commandSizer.Add(self.btnOk, 0, wx.SHAPED | wx.ALIGN_RIGHT)
		self.commandSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED | wx.ALIGN_RIGHT)
		
		self.theSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)
		self.theSizer.Add(self.textSizer, 0, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)
		self.theSizer.Add(self.listSizer, 9, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)		
		self.theSizer.Add(self.commandSizer, 0, wx.EXPAND)
		self.theSizer.Add(wx.StaticText(self, -1, '  '), 0, wx.SHAPED)
		
		self.SetAutoLayout(True)
		self.SetSizerAndFit(self.theSizer)
		
		#/Sizer
		
		#Events:
		
		self.Bind(wx.EVT_BUTTON, self.OnbtnCancel, id=self.ID_CANCEL)
		self.Bind(wx.EVT_BUTTON, self.OnbtnOk, id=self.ID_OK)
		
		self.Bind(wx.EVT_TEXT, self.OnTextChanged, id=self.ID_TXT_MODULE)
				
		self.txtModule.Bind(wx.EVT_CHAR, self.OnTextChar)
		
		self.listModules.Bind(wx.EVT_LISTBOX_DCLICK, self.OnbtnOk)
		self.listModules.Bind(wx.EVT_CHAR, self.OnChar)
		self.Bind(wx.EVT_CHAR, self.OnChar)
						
		#/Events
		
		if self.listModules.GetCount() > 0:
			self.listModules.SetSelection(0)
				
		self.parent.LoadDialogSizeAndPosition(self, 'openimportedmoduledialog.sizeandposition.dat')
	
	def GetSelectedModule(self):
		return self.listModules.GetStringSelection()
	
	def OnCloseW(self, event):
		self.parent.SaveDialogSizeAndPosition(self, 'openimportedmodule.sizeandposition.dat')
		if event is not None:
			event.Skip()	
				
	def OnbtnCancel(self, event):
		self.EndModal(wx.ID_CANCEL)
	
	def OnbtnOk(self, event):
		l = self.listModules.GetCount()
		if l > 0:
			self.EndModal(wx.ID_OK)
		else:
			self.EndModal(wx.ID_CANCEL)			

	def OnTextChar(self, event):
		ekc = event.GetKeyCode()
		i = self.listModules.GetSelection()
		l = self.listModules.GetCount()
		
		if ekc == wx.WXK_UP:
			i -= 1
			if i < 0:
				i = l - 1
			self.listModules.SetSelection(i)					
		elif ekc == wx.WXK_DOWN:
			i += 1
			if i >= l:
				i = 0
			self.listModules.SetSelection(i)		
		self.OnChar(event)
	
	def OnChar(self, event):
		if (event.GetKeyCode() == wx.WXK_ESCAPE):
			self.OnbtnCancel(None)
		elif (event.GetKeyCode() == wx.WXK_RETURN):
			self.OnbtnOk(None)
		else:
			event.Skip()
		
	def OnTextChanged(self, event):
		text = self.txtModule.GetValue()
		stringselection = self.listModules.GetStringSelection()
		l = len(text)
		
		newlist = []
		for mod in self.modules:
			if mod[:l] == text:
				newlist.append(mod)

		self.listModules.Set(newlist)
		
		if stringselection in newlist:
			self.listModules.SetStringSelection(stringselection)
		else:
			if len(newlist) > 0:
				self.listModules.SetSelection(0)
	
		event.Skip()